function [ xax, yax, xfailureP, failureP, failConfidenceTopBottom ] = dataCollector( m, n, runs, cRange, muRange, li, xAxisType , mtxM, sampleM, origTitle, failConfidence, impConf )
%DATACOLLECTOR runs one numerical experiment and outputs the results.  This
%is an internal function only intended to be used by the kARiPlotter
%package.
%
%   Inputs:
%
%   m: The number of desired rows
%
%   n: The number of desired columns
%
%   runs: The number of experiments to perform for each mu value (if
%   xAxisType==1) or for each c value (if xAxisType==2)
%
%   cRange: The c values to be tested
%     If xAxisType==1, then cRange should be a scalar.  If xAxisType==2,
%     then cRange should be a vector.
%
%   li: If xAxisType==2, then these are the leverage scores to be tested.
%     If xAxisType==1, then this will be a function that returns the
%     desired leverage scores.
%
%   muRange: The mu values to be tested
%     If xAxisType==1, then muRange should be a vector.  If xAxisType==2,
%     then muRange should be a scalar.
%
%   xAxisType: A scalar that should be either 1 or 2
%     If xAxisType==1, then we are testing multiple mu values and mu will
%     be on the x-axis of our final plot.  If xAxisType==2, then we are
%     testing multiple c values and c will be on the x-axis of our final
%     plot.
%
%   mtxM: Either a mXn matrix or a function
%     mtxM defines the matrix that we are going to test.  If mtxM is a mXn
%     matrix, then we will use mtxM for our experiments.  If mtxM is a
%     function, then mtxM(m,n,mu) will return the matrix that we will use
%     for our experiments.
%
%   sampleM: A function that defines how we will sample rows from the
%   matrix
%     This function will be passed to the runSampling function.
%
%   origTitle: A string defining the first part of the title for the
%   waitbar.
%
%   failConfidence: A scalar that defined the confidence percent.
%
%   impConf: A boolean variable telling dataCollector whether or nt we are
%   using a new technique to improve the confidence intervals.  This
%   technique will be enabled in a future version of kappa_SQ
%
%   Outputs:
%
%   xax: A vector of the x-values for the desired plot
%
%   yax: A vector of the y-values for the desired plot
%
%   failureP: A vector of the failure percentages for each value of xax
%
%   xfailureP: A vector of the x values for the failure percentage plot
%




%---Code for waitbar---
%we update the waitbar every .01% done, or every 10 seconds, whichever
%comes first
pdone=.01; %the next percent that we will display the percent done
lastTime=0; %last time we updated
wtBr=waitbar(0,origTitle);
%----------------------


skp=0;%this is used to delete the data points where a condn=inf

if xAxisType==1 %if mu is on the x axis
    
    
    if isequal(class(mtxM),'double') %we make sure that mtxM is a function
        error('You cannot test multiple mu with one matrix')
    end
    
    
    yax=zeros(1,length(muRange)*runs);%condition number of sampled data will be stored here
    xax=zeros(size(yax));%xaxis data will be stored here
    failureP=zeros(1,length(muRange));%failure rate data will be stored here
    xfailureP=muRange;
    
    %---Code for waitbar---
    totalWork=runs*cRange*length(muRange);
    %----------------------
    tic
    
    for i=1:length(muRange) %for all coherence values
        
        if isequal(class(li),'double') %see if li is a double or a function
            A=mtxM(m,n,li,muRange(i)); %generate the matrix (A), with the desired leverage scores
        else
            A=mtxM(m,n,li(m,n,muRange(i)),muRange(i)); %generate the matrix (A), with the desired leverage scores
        end
        
        for j=1:runs %do the experiment runs times
            
            yax(j+(i-1)*runs-skp)=runSampling(cRange, A, sampleM);
            %this is the condition number of the sampled matrix
            
            xax(j+(i-1)*runs-skp)=muRange(i);%since the x-axis is coherence, we
            %use the true, measured, value
            
            if isequal(yax(j+(i-1)*runs-skp),inf) %if the sampled matrix is rank deficient
                skp=skp+1;%increase skp so we don't plot this run
                failureP(i)=failureP(i)+1;%add one failure event
            end
            
            %---Code for waitbar---
            curWork=cRange*(i*runs + j);
            if pdone<curWork/totalWork || toc>lastTime+10
                lastTime=toc;
                pdone=pdone+.01;%don't update again until new pdone value
                ttt=toc/curWork*(totalWork-curWork); %approximate time left
                %this may not be very accurate.
                if ttt>3600
                    tLeft=[num2str(ttt/3600),' hours'];
                elseif ttt>60
                    tLeft=[num2str(ttt/60),' minutes'];
                else
                    tLeft=[num2str(ttt),' seconds'];
                end
                waitbar(curWork/totalWork,wtBr,[origTitle,'| ETR: ',tLeft])
            end
            %----------------------
        end
    end
    
elseif xAxisType==2 %if c is on the x axis
    
    yax=zeros(1,length(cRange)*runs);%condition number of sampled data will be stored here
    xax=zeros(size(yax));%xaxis data will be stored here
    failureP=zeros(1,length(cRange));%failure rate data will be stored here
    xfailureP=cRange;
    
    %when xAxisType==2, mtxM is always a matrix
    A=mtxM; %matlab uses pointers, this operation is fast
    
    
    totalWork=runs*sum(cRange);
    tic
    for i=1:length(cRange)
        
        for j=1:runs%do the experiment runs times
            
            yax(j+(i-1)*runs-skp)=runSampling(cRange(i), A, sampleM);
            %this is the condition number of the sampled matrix
            
            xax(j+(i-1)*runs-skp)=cRange(i);
            %This is the number of rows sampled
            
            if isequal(yax(j+(i-1)*runs-skp),inf) %if the sampled matrix is rank deficient
                skp=skp+1;%increase skp so we don't plot this run
                failureP(i)=failureP(i)+1;%add one failure event
            end
            
            %---Code for waitbar---
            curWork=sum(cRange(1:i-1))*runs + cRange(i)*j;
            if pdone<curWork/totalWork || toc>lastTime+10
                lastTime=toc;
                pdone=pdone+.01;%don't update again until new pdone value
                ttt=toc/curWork*(totalWork-curWork);%approximate time left
                %this may not be very accurate.
                if ttt>3600
                    tLeft=[num2str(ttt/3600),' hours'];
                elseif ttt>60
                    tLeft=[num2str(ttt/60),' minutes'];
                else
                    tLeft=[num2str(ttt),' seconds'];
                end
                waitbar(curWork/totalWork,wtBr,[origTitle,'| ETR: ',tLeft])
            end
            %----------------------
        end
    end
else
    error('invalid xAxisType')
end

[~,failConfidenceTopBottom] = binofit(failureP,runs,1-failConfidence);


%%%IMPROVE CONFIDENCE INTERVAL!!!  NOT WRITTEN UP, (but proven)  Note: You
%%%will lose independance
maxNoImprove=10;%If going back or forward the last 10 spots doesn't improve anything, stop .
if impConf
    % for i=1:size(failConfidenceTopBottom,1)
    %     noImproveCnt=0;
    %     %improve top
    %     for j=fliplr(1:i-1)
    %
    %         numRuns=(i-j+1)*runs;
    %         [~,tb]=binofit(sum(failureP(j:i)),numRuns,1-failConfidence);
    %         if tb(2)<failConfidenceTopBottom(i,2)
    %             failConfidenceTopBottom(i,2)=tb(2);
    %         else
    %             noImproveCnt=noImproveCnt+1;
    %             if noImproveCnt>maxNoImprove
    %                 break;
    %             end
    %         end
    %     end
    %
    %     %improve bottom
    %     noImproveCnt=0;
    %     for j=i+1:size(failConfidenceTopBottom,1)
    %         numRuns=(j-i+1)*runs;
    %         [~,tb]=binofit(sum(failureP(i:j)),numRuns,1-failConfidence);
    %         if tb(1)>failConfidenceTopBottom(i,1)
    %             failConfidenceTopBottom(i,1)=tb(1);
    %         else
    %             noImproveCnt=noImproveCnt+1;
    %             if noImproveCnt>maxNoImprove
    %                 break;
    %             end
    %         end
    %
    %     end
    % end
    
    
    %Binom VERSION
    for i=1:size(failConfidenceTopBottom,1)
        
        %improve top
        searchVec=fliplr(1:i-1);
        top=length(searchVec);
        bot=1;
        cur=max(1,ceil((top-bot)/2));
        if ~isempty(searchVec)
            while top~=bot
                
                numRuns=(i-searchVec(cur)+1)*runs;
                [~,tb]=binofit(sum(failureP(searchVec(cur):i)),numRuns,1-failConfidence);
                if tb(2)<failConfidenceTopBottom(i,2)
                    failConfidenceTopBottom(i,2)=tb(2);
                    gb=true;
                else
                    gb=false;
                end
                %         disp('!!!!!!!!!!!!!!')
                % top
                % bot
                % cur
                % gb
                [ top, bot, cur] = binoSearchIter( top, bot, cur, gb );
                %         pause
            end
        end
        
        %improve bottom
        
        searchVec=i+1:size(failConfidenceTopBottom,1);
        top=length(searchVec);
        bot=1;
        cur=max(1,ceil((top-bot)/2));
        
        if ~isempty(searchVec)
            while top~=bot
                
                numRuns=(searchVec(cur)-i+1)*runs;
                [~,tb]=binofit(sum(failureP(i:searchVec(cur))),numRuns,1-failConfidence);
                if tb(1)>failConfidenceTopBottom(i,1)
                    failConfidenceTopBottom(i,1)=tb(1);
                    gb=true;
                else
                    gb=false;
                end
                [ top, bot, cur] = binoSearchIter( top, bot, cur, gb );
            end
        end
    end
    
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


failConfidenceTopBottom=failConfidenceTopBottom'*100;

failureP=failureP/runs*100;%turn failureP into percent

xax=xax(1:end-skp);%remove unused zero entries in the vector
yax=yax(1:end-skp);

%---Code for waitbar---
delete(wtBr)%Done!
%----------------------
end

function [ condn ] = runSampling( cORgamma, A, sampleM )
%RUNSAMPLING returns computes the sampled matrix SQ and returns the
%condition number of SQ, or inf if the sampled matrix isn't full column
%rank. This is an internal function only meant to be used by the kappa_SQ
%package.
%
%   [ condn ] = runSampling( cORgamma, A, sampleM ) returns the
%   condition number of SQ (or inf if SQ is not full column rank)


SQ=sampleM(A, cORgamma);  %<-- this is SQ

% condn=cond(A);

if rank(SQ)<size(A,2) %condn > 10^15
    condn=inf;
else
    condn=cond(SQ);
end



end

