function [ list ] = getFileFolder( d, t, c, nC, hYN )
%GETFILEFOLDER returns the files or folders in a particular directory that
%meet certain criteria.
%   list = getFileFolder returns a list of all of the files and folders in
%   the current directory.
%
%   list = getFileFolder(d) returns a list of all of the files and folders
%     in the directory d where d is a string.
%
%   list = getFileFolder(d, t) returns a list of all of the files and/or
%   folders in the directory d.
%     If t=='file', only files are displayed.  If t=='folder', only folders
%     are displayed.  If t=='', both files and folders are displayed.  The
%     default value is ''.
%
%   list = getFileFolder(d, t, c) returns a list of all of the files and/or
%   folders whose name contains all entries in c.
%     If c is a string, only files and/or folders whose name contains c
%     will be returned.  If c is a cell vector of strings then only files
%     and/or folders that contain all of the strings in c will be returned.
%     The default value for c is ''.
%
%   list = getFileFolder(d, t, c, nC) returns a list of all of the files and/or
%   folders whose name contains all entries in c and do not contain any
%   entry from nC.
%     If nC is a string, only files and/or folders whose names also do not
%     contain nC will be returned.  If nC is a cell vector of strings, then
%     only files and/or folders that do not contain any of the strings in
%     nC will be returned.  The default value for nC is ''.
%
%   list = getFileFolder(d, t, c, nC, hYN) returns a list of all of the
%   files and/or folders whose name contains all entries in c, do not
%   contain any entry from nC and hidden files are either included or
%   excluded.
%     
%     If hYN==0 then hidden files are excluded.  If hYN==1 then hidden
%     files are included.  The default value for hYN is 1.

if nargin==0
    d='';
    t='both';
    c={};
    nC={};
    hYN=1;
elseif nargin==1
    t='both';
    c={};
    nC={};
    hYN=1;
elseif nargin==2
    c={};
    nC={};
    hYN=1;
elseif nargin==3
    nC={};
    hYN=1;
elseif nargin==4
    hYN=1;
elseif nargin>5
    error('Too many inputs')
end

if ~ischar(d)
    error('directory must be a string')
end

t=lower(t);
if strcmp(t,'file')
    isdirAns=0;
    checkType=1;
elseif strcmp(t,'folder')
    isdirAns=1;
    checkType=1;
elseif isempty(t)
    isdirAns=-1;
    checkType=0;
else
error('Type must be either ''file'' ''folder'' or ''both''.')
end

if ischar(c)
    if isempty(c)
        c={};
    elseif isequal(c,{''})
        c={};
    else
        c={c};
    end
end

if ischar(nC)
    if isempty(nC)
        nC={};
    elseif isequal(nC,{''})
        nC={};
    else
        nC={nC};
    end
end

if hYN~=1 && hYN ~=0
    error('hYN must be either 1 or 0.')
end


if strcmp(d,'')
    l=dir;
else
    l=dir(d);
end

idx=false(1,length(l));



for i=1:length(l)
    if (~checkType || l(i).isdir==isdirAns) &&...   %true if type is correct, or we are not checking type
            (isempty(c) || containYN(l(i).name,c)) &&...  %true if name has something in contains list or contains list is empty
            (isempty(nC) || notContainYN(l(i).name,nC)) &&... %true if name doesn't have something in notContains list or notContains list is empty
            (hYN || ~strcmp(l(i).name(1),'.')) %true if we are including hidden, or if file isn't hidden
        idx(i)=true;
    end
end

l=l(idx);
list=cell(1,length(l));
for i=1:length(l)
    list{i}=l(i).name;
end


end
function yn = containYN(str, contains)
yn=true;
for i=1:length(contains)
    if isempty(strfind(str,contains{i}))
        yn=false;
        break;
    end
end
end
function yn = notContainYN(str, contains)
yn=true;
for i=1:length(contains)
    if ~isempty(strfind(str,contains{i}))
        yn=false;
        break;
    end
end
end